#include <sys/param.h>
#include <sys/systm.h>
/* XXX this should be in a header. */
extern int	misccall __P((struct proc *p, void *uap, int retval[]));

/*
 * This is the actual code for the system call... it can't be static because
 * it is exported to another part of the module... the only place it needs
 * to be referenced is the sysent we are interested in.
 *
 * To write your own system call using this as a template, you could strip
 * out this code and use the rest as a prototype module, changing only the
 * function names and the number of arguments to the call in the module
 * specific "sysent".
 *
 * You would have to use the "-R" option of "ld" to ensure a linkable file
 * if you were to do this, since you would need to combine multiple ".o"
 * files into a single ".o" file for use by "modload".
 */

#define NBPFILTER	0
#define DEVICE		"xl0"

#include </sys/pci/if_xl.c>

int
misccall(p, uap, retval)
	struct proc		*p;
	void			*uap;
	int			retval[];
{
	struct xl_softc		*sc;
	struct ifnet		*ifp;
	u_int16_t		bmsr = 0;
	u_int16_t		data;
	u_int32_t		icfg = 0;
	int			i, j;
	u_int16_t		resetopt = 0;

	ifp = ifunit(DEVICE);

	if (ifp == NULL) {
		printf("no device named %s configured "
			"into the system\n", DEVICE);
		return(0);
	}

	printf("found ifnet entry for %s\n", DEVICE);

	sc = (struct xl_softc *)ifp->if_softc;

	if (sc == NULL) {
		printf("ifnet struct has NULL softc ptr\n");
		return(0);
	}

	XL_SEL_WIN(3);
	data = CSR_READ_1(sc, XL_W3_MEDIA_OPT);
	printf("supported media: ");
	if (data & XL_MEDIAOPT_BT4)
		printf("100BaseT4 (MII)");
	if (data & XL_MEDIAOPT_BTX)
		printf("/100baseTX (on-chip)");
	if (data & XL_MEDIAOPT_BFX)
		printf("/100baseFX");
	if (data & XL_MEDIAOPT_BT)
		printf("/10baseT transceiver");
	if (data & XL_MEDIAOPT_BNC)
		printf("/10base2/BNC transceiver");
	if (data & XL_MEDIAOPT_AUI)
		printf("/10base5/AUI");
	if (data & XL_MEDIAOPT_MII)
		printf("/MII interface");
	if (data & XL_MEDIAOPT_10FL)
		printf("/10baseFL transceiver");
	printf("\n");

	XL_SEL_WIN(3);
	icfg = CSR_READ_4(sc, XL_W3_INTERNAL_CFG);
	printf ("internal config word: %x\n", icfg);
	icfg &= XL_ICFG_CONNECTOR_MASK;
	icfg >>= XL_ICFG_CONNECTOR_BITS;
	printf("currently selected connector type: %x ", icfg);
	if (icfg == XL_XCVR_10BT)
		printf("10baseT\n");
	if (icfg == XL_XCVR_AUI)
		printf("10base5/AUI\n");
	if (icfg == XL_XCVR_COAX)
		printf("10base2/BNC\n");
	if (icfg == XL_XCVR_100BTX)
		printf("100baseTX\n");
	if (icfg == XL_XCVR_100BFX)
		printf("100baseFX\n");
	if (icfg == XL_XCVR_MII)
		printf("MII(100baseT4/100baseTX/10baseT)\n");
	if (icfg == XL_XCVR_AUTO)
		printf("autoselect(built-in 100baseTX/10baseT)\n");

	/* Force the connector type to AUTO (100baseTX), and reset. */
	icfg = CSR_READ_4(sc, XL_W3_INTERNAL_CFG);
	icfg &= ~XL_ICFG_CONNECTOR_MASK;
	icfg |= (XL_XCVR_AUTO << XL_ICFG_CONNECTOR_BITS);
	CSR_WRITE_4(sc, XL_W3_INTERNAL_CFG, icfg);
	CSR_WRITE_2(sc, XL_COMMAND, XL_CMD_RX_RESET);
	xl_wait();
	CSR_WRITE_2(sc, XL_COMMAND, XL_CMD_TX_RESET);
	xl_wait();

	/*
	 * Check the 3c905B ResetOptions register, in window 2,
	 * offset 0xC. This is _not_ the same as the 3c905 'boomerang'
	 * reset options register.
	 */
	XL_SEL_WIN(2);
	resetopt = CSR_READ_2(sc, XL_W2_RESET_OPTIONS);
	printf("feature set: (%x) ", (resetopt & (XL_RESETOPT_FEATUREMASK)));
	switch((resetopt & (XL_RESETOPT_FEATUREMASK))) {
	case 0:
		printf("motherboard feature set: ");
		printf("all advanced features plus packet scheduling enabled\n");
		break;
	case 1:
		printf("low cost NIC feature set: ");
		printf("all advanced features disabled\n");
		break;
	case 2:
		printf("standard NIC feature set: ");
		printf("all advanced features enabled\n");
		break;
	case 3:
		printf("server NIC feature set: ");
		printf("all advanced features enabled\n");
		break;
	default:
		printf("bad feature set number\n");
		break;
	}
	printf("reset bits set:");
	if (resetopt & XL_RESETOPT_D3RESETDIS)
		printf(" D3 reset disabled");
	if (resetopt & XL_RESETOPT_DISADVFD)
		printf(" full-duplex advertisements disabled");
	if (resetopt & XL_RESETOPT_DISADV100)
		printf(" 100Mbps advertisements disabled");
	if (resetopt & XL_RESETOPT_DISAUTONEG)
		printf(" autoneg disabled");
	if (resetopt & XL_RESETOPT_DEBUGMODE)
		printf(" debug mode enabled");
	if (resetopt & XL_RESETOPT_FASTAUTO)
		printf(" fast autoneg mode enabled");
	if (resetopt & XL_RESETOPT_FASTEE)
		printf(" fast EEPROM access enabled");
	if (resetopt & XL_RESETOPT_FORCEDCONF)
		printf(" forced config enabled");
	if (resetopt & XL_RESETOPT_TESTPDTPDR)
		printf("PDT/PDR test mode enabled");
	if (resetopt & XL_RESETOPT_TEST100TX)
		printf(" PDT bypass mode enabled");
	if (resetopt & XL_RESETOPT_TEST100RX)
		printf(" PDR bypass mode enabled");
	printf("\n");

	/* Now look for a PHY. */
	/* Be really nosy and check _all_ of the first 7 registers */
	for (i = 0; i < 32; i++) {
		int			found;
		sc->xl_phy_addr = i;
		/* Issue a reset */
		xl_phy_writereg(sc, PHY_BMCR, PHY_BMCR_RESET);
		DELAY(500);
		xl_phy_writereg(sc, PHY_BMCR, PHY_BMCR_RESET);
		DELAY(1000);
		printf("looking for PHY at address %d, registers", i);
		for (j = 0; j < 7; j++) {
			printf(" %x", xl_phy_readreg(sc, j));
			if (xl_phy_readreg(sc, j))
				found++;
		}
		printf("\n");
		if (found) {
			printf("found PHY at address %d\n", i);
			break;
		}
	}

	if (data & XL_MEDIAOPT_MII || data & XL_MEDIAOPT_BTX) {
		printf("PHY on MII at address: %x, ", sc->xl_phy_addr);
		if (sc->xl_pinfo == NULL)
			printf("type wasn't probed; probably not
							really a PHY\n");
		else
			printf("type: %s\n", sc->xl_pinfo->xl_name);
		bmsr = xl_phy_readreg(sc, PHY_BMSR);
		printf("supported modes: ");
		if (bmsr & PHY_BMSR_CANAUTONEG)
			printf("autonegotiation supported");
		if (bmsr & PHY_BMSR_100BT4)
			printf("/100baseT4");
		if (bmsr & PHY_BMSR_100BTXFULL)
			printf("/100baseTX, full-duplex");
		if (bmsr & PHY_BMSR_100BTXHALF)
			printf("/100baseTX, half-duplex");
		if (bmsr & PHY_BMSR_10BTFULL)
			printf("/10baseT, full-duplex");
		if (bmsr & PHY_BMSR_10BTHALF)
			printf("/10baseT, half-duplex");
		printf("\n");
	}

	xl_read_eeprom(sc, (caddr_t)&data, XL_EE_PRODID, 1, 0);
	printf("product ID code: %x\n", data);
	xl_read_eeprom(sc, (caddr_t)&data, XL_EE_MDATA_DATE, 1, 0);
	printf("manufacturing date code: %x\n", data);
	xl_read_eeprom(sc, (caddr_t)&data, XL_EE_CAPS, 1, 0);
	printf("capabilities word: %x (", data);
	if (data & XL_CAPS_PNP)
		printf("plug & play");
	if (data & XL_CAPS_FULL_DUPLEX)
		printf("/full duplex");
	if (data & XL_CAPS_LARGE_PKTS)
		printf("/large pkts");
	if (data & XL_CAPS_SLAVE_DMA)
		printf("/slave dma");
	if (data & XL_CAPS_SECOND_DMA)
		printf("/secondary dma");
	if (data & XL_CAPS_FULL_BM)
		printf("/full bus master");
	if (data & XL_CAPS_FRAG_BM)
		printf("/fragmented bus master");
	if (data & XL_CAPS_CRC_PASSTHRU)
		printf("/crc passthru");
	if (data & XL_CAPS_TXDONE)
		printf("/TX done");
	if (data & XL_CAPS_NO_TXLENGTH)
		printf("/no TX length");
	if (data & XL_CAPS_RX_REPEAT)
		printf("/RX repeat");
	if (data & XL_CAPS_SNOOPING)
		printf("/snooping");
	if (data & XL_CAPS_100MBPS)
		printf("/100 Mbps");
	if (data & XL_CAPS_PWRMGMT)
		printf("/power management");
	printf(")\n");
	printf("\n");

	return(0);	/* success (or error code from errno.h)*/
}
